#include "c4d.h"
#include "lib_browser.h"
#include "main.h"

// be sure to use a unique ID obtained from www.plugincafe.com
#define ID_GET_CONTENT_BROWSER 1040598

class COFFEEFilterContentBrowserIterator : public CommandData
{

private:
	Bool CreateBaseContainer();
	void RecurseHierarchy(SDKBrowserContentNodeRef op);
	Bool AddDataToBc(SDKBrowserURL presetPath);
	BaseContainer* GetBaseContainer();
	Bool IsC4DContentBrowserLibrary(SDKBrowserURL presetPath);

public:
	virtual Int32 GetState(BaseDocument* doc) { return CMD_ENABLED; }
	virtual Bool Execute(BaseDocument* doc);

};

// Build a basic BaseContainer stored in the active document
Bool COFFEEFilterContentBrowserIterator::CreateBaseContainer()
{
	BaseDocument* doc = GetActiveDocument();
	if (!doc) return false;

	BaseContainer* bcDoc = doc->GetDataInstance();
	if (!bcDoc) return false;
	
	BaseContainer bcData = BaseContainer();
	bcDoc->SetContainer(ID_GET_CONTENT_BROWSER, bcData);
	return true;
}

// Check if it's a C4D Presets
Bool COFFEEFilterContentBrowserIterator::IsC4DContentBrowserLibrary(SDKBrowserURL presetPath)
{
	String fn = presetPath.GetString();
	maxon::BaseArray<String> array;

	if (array.Resize(7) == maxon::FAILED)
		return false;

	array[0] = "bodypaint 3d.lib4d";
	array[1] = "broadcast.lib4d";
	array[2] = "default presets.lib4d";
	array[3] = "Prime.lib4d";
	array[4] = "sculpting.lib4d";
	array[5] = "studio.lib4d";
	array[6] = "visualize.lib4d";

	for (Int i = 0; i < array.GetCount(); i++)
	{
		if (fn.FindFirst(array[i], nullptr))
			return true;
	}

	return false;
}

// Iterate over the sdk browser
void COFFEEFilterContentBrowserIterator::RecurseHierarchy(SDKBrowserContentNodeRef op)
{
	while (op)
	{
		if (IsC4DContentBrowserLibrary(op->GetNodeURL()))
		{
			op = op->GetNext();
		}
		else
		{
			if (op->GetTypeID() == 3)
			{
				String test = op->GetTypeName(1);
				if (test == "File Preset" || test == "Object Preset") {
					AddDataToBc(op->GetNodeURL());
				}
			}
			RecurseHierarchy(op->GetDown());
			op = op->GetNext();
		}
	}
}

// Add a SDKBrowserURL to the BaseContainer stored in the active document
Bool COFFEEFilterContentBrowserIterator::AddDataToBc(SDKBrowserURL presetPath)
{
	BaseContainer* bc = GetBaseContainer();
	if (!bc) return false;

	String str = presetPath.GetString();
	Filename fn = Filename(str);
	String suffix = fn.GetSuffix();
	if (suffix != "c4d" && suffix != "")
		return true;

	Int32 i = 0, cnt = 0;
	while (true)
	{
		GeData *data = bc->GetIndexData(i++);
		if (data == nullptr) break;
		cnt++;
	}

	bc->SetString(cnt, str);
	StatusSetText(str);

	return true;
}

// retrieve the BaseContainer previously created by CreateBaseContainer
BaseContainer* COFFEEFilterContentBrowserIterator::GetBaseContainer()
{
	BaseDocument* doc = GetActiveDocument();
	if (!doc) return nullptr;

	BaseContainer* bcDoc = doc->GetDataInstance();
	if (!bcDoc) return nullptr;

	BaseContainer* bcData = bcDoc->GetContainerInstance(ID_GET_CONTENT_BROWSER);
	if (!bcData) return nullptr;

	return bcData;
}

Bool COFFEEFilterContentBrowserIterator::Execute(BaseDocument * doc)
{
	if (!doc) return false;
	CreateBaseContainer();

	SDKBrowserContentNodeRef ref = SDKBrowser::GetCategoryNode(SDKBrowser::CategoryPresetNodes);
	if (!ref) return false;

	RecurseHierarchy(ref);
	StatusClear();
	SpecialEventAdd(ID_GET_CONTENT_BROWSER);
	return true;
}


Bool RegisterCOFFEEFilterContentBrowserIterator()
{
	if (!FindPlugin(ID_GET_CONTENT_BROWSER, PLUGINTYPE_COMMAND))
		return RegisterCommandPlugin(ID_GET_CONTENT_BROWSER , String("Content Browser"), PLUGINFLAG_HIDE, nullptr, String(""), NewObjClear(COFFEEFilterContentBrowserIterator));

	return true;
}
