"""
CINEMA 4D MAXON LABS
(c) MAXON Computer GmbH, all rights reserved

Author : Adam Maxime
Email : m_adam@maxon.de

Class that store all our constants.
CONSTANT CANNOT BE ADDED AT RUN TIME !
This class is immutable.

Basic Usage:
    print Const.PLUGIN_ID
"""

import c4d

class Singleton(type):
    _instances = {}

    def __call__(cls, *args, **kwargs):
        if cls not in cls._instances:
            cls._instances[cls] = super(Singleton, cls).__call__(*args, **kwargs)
        return cls._instances[cls]

class Const(object):
    __slots__ = [] #make immutable
    __metaclass__ = Singleton

    # make it strongly immutable
    def __setattr__(self, key, value):
        raise Warning("You are not allowed to add constant at runtime !")

    ID_GET_CONTENT_BROWSER = 1040598
    PLUGIN_ID = 1040533
    VERSION = 1.01

    CURRENT_C4D_VERSION = float(str(c4d.GetC4DVersion())[:2] + "." + str(c4d.GetC4DVersion())[2:])

    BC_IS_CONVERTED_DOC = 0

    DIALOG_MENU_ADD_CURRENT_DOC = 1000
    DIALOG_MENU_ADD_ALL_OPEN_DOCS = 1001
    DIALOG_MENU_CONTENT_BROWSER = 1002
    DIALOG_MENU_FILE = 1003
    DIALOG_MENU_FOLDER = 1004
    DIALOG_MENU_ONE_SUBFOLDER = 1005
    DIALOG_MENU_ALL_SUBFOLDER = 1006

    DIALOG_MENU_SELECT_ALL = 1010
    DIALOG_MENU_SELECT_NEED_CONVERTION = 1011
    DIALOG_MENU_SELECT_CONVERTED = 1012
    DIALOG_MENU_SELECT_NO_NEED_CONVERTION = 1013
    DIALOG_MENU_SELECT_PRESET = 1014

    DIALOG_MENU_DESELECT_ALL = 1020
    DIALOG_MENU_DESELECT_NEED_CONVERTION = 1021
    DIALOG_MENU_DESELECT_CONVERTED = 1022
    DIALOG_MENU_DESELECT_NO_NEED_CONVERTION = 1023
    DIALOG_MENU_DESELECT_PRESET = 1024

    DIALOG_MENU_REMOVE_SELECT = 1030
    DIALOG_MENU_REMOVE_NEED_CONVERTION = 1031
    DIALOG_MENU_REMOVE_CONVERTED = 1032
    DIALOG_MENU_REMOVE_NO_NEED_CONVERTION = 1033
    DIALOG_MENU_REMOVE_PRESET = 1034

    DIALOG_CONVERT_ALL_SELECTED_DOC = 1100

    MODE_FILE = 0
    MODE_FOLDER = 1
    MODE_ONEFOLDER = 2
    MODE_SUBFOLDER = 3

    MODE_CHECK = 0
    MODE_CONVERT = 1

    MODE_PRE12 = 0
    MODE_CONVERTED = 1
    MODE_POST12 = 2
    MODE_ALL = 3

    DIALOG_FILTER_GRP = 1200
    DIALOG_FILTER_TEXT = 1202
    DIALOG_FILTER_CONVERTED = 1203
    DIALOG_FILTER_NO_CONV = 1204
    DIALOG_FILTER_NEED_CONV = 1205
    DIALOG_CLEAR_ALL_DOC = 1206


    DIALOG_BASE_BOTTOM_TEXT_NEED_UPDATE = 1992
    DIALOG_BASE_BOTTOM_TEXT_ALREADY_CONVERTED = 1993
    DIALOG_BASE_BOTTOM_TEXT_GOOD = 1994
    DIALOG_BASE_BOTTOM_GROUP = 1995
    DIALOG_BASE_MAIN_GROUP_UI = 1996
    DIALOG_BASE_GROUP_PATH = 1997
    DIALOG_BASE_GROUP_UI = 1998
    DIALOG_BASE_GROUP_UI_SCROLL = 1999
    DIALOG_BASE_UI = 2000

    UI_FILE_GROUP = 1000
    UI_FILE_GROUP_SCROLL = 1001
    UI_FILE_GROUP_HEADER = 1002
    UI_FILE_CHECKBOX = 1003
    UI_FILE_PATH = 1004
    UI_FILE_GROUP_CONVERTED = 1005
    UI_FILE_GROUP_FILE_CONVERTED = 1006
    UI_FILE_CONVERT_BUTTON = 1007
    UI_FILE_REFRESH_BUTTON = 1008
    UI_FILE_OPEN_BUTTON = 1009
    UI_FILE_EXPLORER_BUTTON = 1010
    UI_FILE_CLEAR_BUTTON = 1011

    UI_COLOR_GOOD = c4d.Vector(0.297, 0.72, 0.13)
    UI_COLOR_ALREADY_CONVERTED = c4d.Vector(0.76, 0.85, 0.55)
    UI_COLOR_NEED_UPDATE = c4d.Vector(0.9, 0.75, 0)
